<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:xsd="http://www.w3.org/2001/XMLSchema"
                xmlns:saxon="http://saxon.sf.net/"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:schold="http://www.ascc.net/xml/schematron"
                xmlns:iso="http://purl.oclc.org/dsdl/schematron"
                xmlns:xhtml="http://www.w3.org/1999/xhtml"
                version="2.0"><!--Importing stylesheet additions-->
   <xsl:output xmlns:sch="http://www.ascc.net/xml/schematron" method="text"/>
   <!--Implementers: please note that overriding process-prolog or process-root is 
    the preferred method for meta-stylesheets to use where possible. -->
   <xsl:param name="archiveDirParameter"/>
   <xsl:param name="archiveNameParameter"/>
   <xsl:param name="fileNameParameter"/>
   <xsl:param name="fileDirParameter"/>
   <xsl:variable name="document-uri">
      <xsl:value-of select="document-uri(/)"/>
   </xsl:variable>

   <!--PHASES-->


   <!--PROLOG-->


   <!--XSD TYPES FOR XSLT2-->


   <!--KEYS AND FUNCTIONS-->


   <!--DEFAULT RULES-->


   <!--MODE: SCHEMATRON-SELECT-FULL-PATH-->
   <!--This mode can be used to generate an ugly though full XPath for locators-->
   <xsl:template match="*" mode="schematron-select-full-path">
      <xsl:apply-templates select="." mode="schematron-get-full-path"/>
   </xsl:template>

   <!--MODE: SCHEMATRON-FULL-PATH-->
   <!--This mode can be used to generate an ugly though full XPath for locators-->
   <xsl:template match="*" mode="schematron-get-full-path">
      <xsl:apply-templates select="parent::*" mode="schematron-get-full-path"/>
      <xsl:text>/</xsl:text>
      <xsl:choose>
         <xsl:when test="namespace-uri()=''">
            <xsl:value-of select="name()"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:text>*:</xsl:text>
            <xsl:value-of select="local-name()"/>
            <xsl:text>[namespace-uri()='</xsl:text>
            <xsl:value-of select="namespace-uri()"/>
            <xsl:text>']</xsl:text>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:variable name="preceding"
                    select="count(preceding-sibling::*[local-name()=local-name(current())                                   and namespace-uri() = namespace-uri(current())])"/>
      <xsl:text>[</xsl:text>
      <xsl:value-of select="1+ $preceding"/>
      <xsl:text>]</xsl:text>
   </xsl:template>
   <xsl:template match="@*" mode="schematron-get-full-path">
      <xsl:apply-templates select="parent::*" mode="schematron-get-full-path"/>
      <xsl:text>/</xsl:text>
      <xsl:choose>
         <xsl:when test="namespace-uri()=''">@<xsl:value-of select="name()"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:text>@*[local-name()='</xsl:text>
            <xsl:value-of select="local-name()"/>
            <xsl:text>' and namespace-uri()='</xsl:text>
            <xsl:value-of select="namespace-uri()"/>
            <xsl:text>']</xsl:text>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:template>

   <!--MODE: SCHEMATRON-FULL-PATH-2-->
   <!--This mode can be used to generate prefixed XPath for humans-->
   <xsl:template match="node() | @*" mode="schematron-get-full-path-2">
      <xsl:for-each select="ancestor-or-self::*">
         <xsl:text>/</xsl:text>
         <xsl:value-of select="name(.)"/>
         <xsl:if test="preceding-sibling::*[name(.)=name(current())]">
            <xsl:text>[</xsl:text>
            <xsl:value-of select="count(preceding-sibling::*[name(.)=name(current())])+1"/>
            <xsl:text>]</xsl:text>
         </xsl:if>
      </xsl:for-each>
      <xsl:if test="not(self::*)">
         <xsl:text/>/@<xsl:value-of select="name(.)"/>
      </xsl:if>
   </xsl:template>
   <!--MODE: SCHEMATRON-FULL-PATH-3-->
   <!--This mode can be used to generate prefixed XPath for humans 
	(Top-level element has index)-->
   <xsl:template match="node() | @*" mode="schematron-get-full-path-3">
      <xsl:for-each select="ancestor-or-self::*">
         <xsl:text>/</xsl:text>
         <xsl:value-of select="name(.)"/>
         <xsl:if test="parent::*">
            <xsl:text>[</xsl:text>
            <xsl:value-of select="count(preceding-sibling::*[name(.)=name(current())])+1"/>
            <xsl:text>]</xsl:text>
         </xsl:if>
      </xsl:for-each>
      <xsl:if test="self::attribute()">
         <xsl:text/>/@<xsl:value-of select="name(.)"/>
      </xsl:if>
      <xsl:if test="self::text()">
         <xsl:text/>/text()"/&gt;
		</xsl:if>
      <xsl:if test="self::processing-instruction()">
         <xsl:text/>/processing-instruction(<xsl:value-of select="name(.)"/>
         <xsl:text>)</xsl:text>
      </xsl:if>
   </xsl:template>

   <!--MODE: GENERATE-ID-FROM-PATH -->
   <xsl:template match="/" mode="generate-id-from-path"/>
   <xsl:template match="text()" mode="generate-id-from-path">
      <xsl:apply-templates select="parent::*" mode="generate-id-from-path"/>
      <xsl:value-of select="concat('.text-', 1+count(preceding-sibling::text()), '-')"/>
   </xsl:template>
   <xsl:template match="comment()" mode="generate-id-from-path">
      <xsl:apply-templates select="parent::*" mode="generate-id-from-path"/>
      <xsl:value-of select="concat('.comment-', 1+count(preceding-sibling::comment()), '-')"/>
   </xsl:template>
   <xsl:template match="processing-instruction()" mode="generate-id-from-path">
      <xsl:apply-templates select="parent::*" mode="generate-id-from-path"/>
      <xsl:value-of select="concat('.processing-instruction-', 1+count(preceding-sibling::processing-instruction()), '-')"/>
   </xsl:template>
   <xsl:template match="@*" mode="generate-id-from-path">
      <xsl:apply-templates select="parent::*" mode="generate-id-from-path"/>
      <xsl:value-of select="concat('.@', name())"/>
   </xsl:template>
   <xsl:template match="*" mode="generate-id-from-path" priority="-0.5">
      <xsl:apply-templates select="parent::*" mode="generate-id-from-path"/>
      <xsl:text>.</xsl:text>
      <xsl:value-of select="concat('.',name(),'-',1+count(preceding-sibling::*[name()=name(current())]),'-')"/>
   </xsl:template>

   <!--MODE: GENERATE-ID-2 -->
   <xsl:template match="/" mode="generate-id-2">U</xsl:template>
   <xsl:template match="*" mode="generate-id-2" priority="2">
      <xsl:text>U</xsl:text>
      <xsl:number level="multiple" count="*"/>
   </xsl:template>
   <xsl:template match="node()" mode="generate-id-2">
      <xsl:text>U.</xsl:text>
      <xsl:number level="multiple" count="*"/>
      <xsl:text>n</xsl:text>
      <xsl:number count="node()"/>
   </xsl:template>
   <xsl:template match="@*" mode="generate-id-2">
      <xsl:text>U.</xsl:text>
      <xsl:number level="multiple" count="*"/>
      <xsl:text>_</xsl:text>
      <xsl:value-of select="string-length(local-name(.))"/>
      <xsl:text>_</xsl:text>
      <xsl:value-of select="translate(name(),':','.')"/>
   </xsl:template>
   <!--Strip characters-->
   <xsl:template match="text()" priority="-1"/>

   <!--SCHEMA SETUP-->
   <xsl:template match="/"><!--Root node processing collects all assertions-->
      <xsl:variable xmlns:sch="http://www.ascc.net/xml/schematron" name="result">
         <xsl:apply-templates select="/" mode="M0"/>
      </xsl:variable>
      <xsl:if xmlns:sch="http://www.ascc.net/xml/schematron"
              test="normalize-space(string($result))">
         <xsl:message terminate="yes">
            <xsl:value-of select="string($result)" disable-output-escaping="yes"/>
         </xsl:message>
      </xsl:if>
   </xsl:template>

   <!--SCHEMATRON PATTERNS-->


   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="/" priority="1002" mode="M0">

		<!--REPORT -->
      <xsl:if test="adoption">ERROR: At this time the document element &lt;adoption&gt; is not supported<xsl:text xmlns:sch="http://www.ascc.net/xml/schematron">: </xsl:text>
         <xsl:apply-templates xmlns:sch="http://www.ascc.net/xml/schematron"
                              select="."
                              mode="schematron-get-full-path-3"/>
         <xsl:text xmlns:sch="http://www.ascc.net/xml/schematron" xml:space="preserve">
</xsl:text>
      </xsl:if>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="standard"/>
         <xsl:otherwise>ERROR: At this time only documents with the document element &lt;standard&gt; are supported. This document has &lt;<xsl:text/>
            <xsl:value-of select="name(*)"/>
            <xsl:text/>&gt; as the document element<xsl:text xmlns:sch="http://www.ascc.net/xml/schematron">: </xsl:text>
            <xsl:apply-templates xmlns:sch="http://www.ascc.net/xml/schematron"
                                 select="."
                                 mode="schematron-get-full-path-3"/>
            <xsl:text xmlns:sch="http://www.ascc.net/xml/schematron" xml:space="preserve">
</xsl:text>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="node()" mode="M0"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="front" priority="1001" mode="M0">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="exists(iso-meta) or exists(reg-meta) or                     exists(nat-meta) or exists(std-doc-meta) or                     exists(std-meta)"/>
         <xsl:otherwise>ERROR: The input document has no front meta data of any kind.<xsl:text xmlns:sch="http://www.ascc.net/xml/schematron">: </xsl:text>
            <xsl:apply-templates xmlns:sch="http://www.ascc.net/xml/schematron"
                                 select="."
                                 mode="schematron-get-full-path-3"/>
            <xsl:text xmlns:sch="http://www.ascc.net/xml/schematron" xml:space="preserve">
</xsl:text>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="node()" mode="M0"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="iso-meta | reg-meta | nat-meta | std-doc-meta"
                 priority="1000"
                 mode="M0">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="exists((doc-ref[normalize-space(.)],                             std-ref[not(@type)],                             std-ref[@type='dated'],                             std-ref[@type='short'],                             std-ident/doc-number)[1])"/>
         <xsl:otherwise>ERROR: There is no document reference information associated with the metadata<xsl:text xmlns:sch="http://www.ascc.net/xml/schematron">: </xsl:text>
            <xsl:apply-templates xmlns:sch="http://www.ascc.net/xml/schematron"
                                 select="."
                                 mode="schematron-get-full-path-3"/>
            <xsl:text xmlns:sch="http://www.ascc.net/xml/schematron" xml:space="preserve">
</xsl:text>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="node()" mode="M0"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M0"/>
   <xsl:template match="@*|node()" priority="-2" mode="M0">
      <xsl:apply-templates select="node()" mode="M0"/>
   </xsl:template>
</xsl:stylesheet>
